#!/usr/bin/perl -w
  
use strict;
use warnings;
use Device::SerialPort;
 
my $file;
my $line;
my $lastchar;
my $rrddb = "/var/apps/geiger/geiger.rrd"; 

my $port = Device::SerialPort->new("/dev/ttyS0");
$port->databits(8);
$port->baudrate(9600);
$port->parity("none");
$port->stopbits(1);
$port->write_settings || undef $port;
$port->read_const_time(0);
 
my $log='/var/apps/geiger/probe.log';
open(LOG, ">>", $log) or die("Could not open file.");
my $offcut = "";

while(1){
	my $timestamp = `date +%s`;
	my $delay = 60 - ($timestamp % 60);
	$timestamp += $delay;
	print LOG "sleeping for $delay seconds\n";
	sleep $delay;
	my $count = 0;
	    while(1) {
		$line = &readline;
		if (length($line) == 0) {
		     last;
		}
		# restore incomplete last line from previous readline
                $line = $offcut . $line;
                $offcut = "";
		$lastchar = substr($line, -1, 1);
		#print "lastchar is <$lastchar>\n";
		#check for new incomplete line
		if ($lastchar ne "\n" && $line !~ /(SLOW|FAST)$/) {
		    # save incomplete last line, and skip
                    $offcut = $line;
		    print LOG "incomplete line <$line>\n";
		    next;
		}
		chomp $line;
		# discard empty or malformed lines
		if (length($line) == 0 ) {
		    next;
		}
		print $line;
		my @values = split(',', $line);
		my $arraysize = @values;
		if ($arraysize != 7) {
		    next;
		}

#		print LOG "$line has $arraysize values, ";
		if ($values[6] =~ /SLOW|FAST/) {
		    if (defined $values[1]) {
			$count += $values[1];
		    }
		} else {
		    print LOG "unexpected row type: $values[6], ";
		    $count = 1000;
		}
	    }
	    $timestamp = `date +%s`;
	    chomp $timestamp;
	    print LOG "time=$timestamp, returned count = $count \n";
	    my $result = `rrdtool updatev $rrddb $timestamp:$count`;
	    print LOG "updatev returned: $result";
}
$port->close();
close(LOG);

sub readline {
	my $line = "";
	my $char;
	while (1) {
		$char = $port->read(1);
		if (length($char) == 0 || $char eq "\r") {
			return $line;
		}
		$line .= $char;
	}
}
